@@ -18,6 +18,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 //
 //
 // Author(s)     : Sylvain Pion, Michael Hemmer
@@ -82,7 +83,7 @@ public:
     : _inf(i), _sup(i) {}
 
   Interval_nt(long long i)
-    : _inf((double)i), _sup((double)i)
+    : _inf(static_cast<double>(i)), _sup(static_cast<double>(i))
   {
     // gcc ignores -frounding-math when converting integers to floats.
 #ifdef __GNUC__
@@ -94,7 +95,7 @@ public:
   }
 
   Interval_nt(unsigned long long i)
-    : _inf((double)i), _sup((double)i)
+    : _inf(static_cast<double>(i)), _sup(static_cast<double>(i))
   {
 #ifdef __GNUC__
     unsigned long long safe = 1ULL << 52; // Use numeric_limits?
@@ -177,7 +178,7 @@ public:
 
   static IA largest()
   {
-    return IA(-internal::infinity, internal::infinity);
+    return IA(-std::numeric_limits<double>::infinity(), std::numeric_limits<double>::infinity());
   }
 
   static IA smallest()
@@ -213,13 +214,13 @@ private:
   };
 
 #ifndef CGAL_DISABLE_ROUNDING_MATH_CHECK
-  static Test_runtime_rounding_modes tester;
+  static const Test_runtime_rounding_modes tester;
 #endif
 };
 
 #ifndef CGAL_DISABLE_ROUNDING_MATH_CHECK
 template <bool Protected>
-typename Interval_nt<Protected>::Test_runtime_rounding_modes
+const typename Interval_nt<Protected>::Test_runtime_rounding_modes
 Interval_nt<Protected>::tester;
 #endif
 
@@ -445,7 +446,7 @@ relative_precision(const Interval_nt<Protected> & d)
 
 template< bool Protected >
 class Is_valid< Interval_nt<Protected> >
-  : public std::unary_function< Interval_nt<Protected>, bool > {
+  : public CGAL::cpp98::unary_function< Interval_nt<Protected>, bool > {
   public :
     bool operator()( const Interval_nt<Protected>& x ) const {
       return is_valid(x.inf()) &&
@@ -463,7 +464,7 @@ std::ostream & operator<< (std::ostream &os, const Interval_nt<Protected> & I )
 #define CGAL_SWALLOW(IS,CHAR)                           \
     {                                                   \
         char c;                                         \
-        do c = is.get(); while (isspace(c));            \
+        do is.get(c); while (isspace(c));            \
         if (c != CHAR) {                                \
             is.setstate(std::ios_base::failbit);        \
         }                                               \
@@ -473,7 +474,7 @@ template <bool Protected>
 std::istream & operator>> (std::istream &is, Interval_nt<Protected> & I)
 {
     char c;
-    do c = is.get(); while (isspace(c));
+    do is.get(c); while (isspace(c));
     is.putback(c);
     if(c == '['){ // read original output from operator <<
         double inf,sup;
@@ -682,7 +683,7 @@ operator/ (const Interval_nt<Protected> & a, double b)
 // TODO: What about these two guys? Where do they belong to?
 template <bool Protected>
 struct Min <Interval_nt<Protected> >
-    : public std::binary_function<Interval_nt<Protected>,
+    : public CGAL::cpp98::binary_function<Interval_nt<Protected>,
                              Interval_nt<Protected>,
                              Interval_nt<Protected> >
 {
@@ -697,7 +698,7 @@ struct Min <Interval_nt<Protected> >
 
 template <bool Protected>
 struct Max <Interval_nt<Protected> >
-    : public std::binary_function<Interval_nt<Protected>,
+    : public CGAL::cpp98::binary_function<Interval_nt<Protected>,
                              Interval_nt<Protected>,
                              Interval_nt<Protected> >
 {
@@ -744,7 +745,7 @@ ldexp(const Interval_nt<b> &i, int e)
 // TODO: To which concept do these functors belong? Can we remove them??
 template < bool b >
 struct Equal_to < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -753,7 +754,7 @@ struct Equal_to < Interval_nt<b>, Interval_nt<b> >
 
 template < bool b >
 struct Not_equal_to < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -762,7 +763,7 @@ struct Not_equal_to < Interval_nt<b>, Interval_nt<b> >
 
 template < bool b >
 struct Greater < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -771,7 +772,7 @@ struct Greater < Interval_nt<b>, Interval_nt<b> >
 
 template < bool b >
 struct Less < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -780,7 +781,7 @@ struct Less < Interval_nt<b>, Interval_nt<b> >
 
 template < bool b >
 struct Greater_equal < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -789,7 +790,7 @@ struct Greater_equal < Interval_nt<b>, Interval_nt<b> >
 
 template < bool b >
 struct Less_equal < Interval_nt<b>, Interval_nt<b> >
-  : public std::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
+  : public CGAL::cpp98::binary_function< Interval_nt<b>, Interval_nt<b>, Uncertain<bool> >
 {
   Uncertain<bool> operator()( const Interval_nt<b>& x,
                               const Interval_nt<b>& y) const
@@ -930,23 +931,6 @@ namespace INTERN_INTERVAL_NT {
     return Uncertain<bool>::indeterminate();
   }
 
- // TODO: Whats this for? Why is this in this file??
-  inline
-  std::pair<double, double>
-  to_interval (const long & l)
-  {
-    if (sizeof(double) > sizeof(long)) {
-      // On 64bit platforms, a long doesn't fit exactly in a double.
-      // Well, a perfect fix would be to use std::numeric_limits<>, but...
-      Protect_FPU_rounding<true> P(CGAL_FE_TONEAREST);
-      Interval_nt<false> approx ((double) l);
-      FPU_set_cw(CGAL_FE_UPWARD);
-      approx += Interval_nt<false>::smallest();
-      return approx.pair();
-    }
-    else
-      return std::pair<double,double>(l,l);
-  }
 } // namespace INTERN_INTERVAL_NT
 
 
@@ -959,7 +943,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
   typedef Uncertain<CGAL::Comparison_result> Comparison_result; 
 
     class Abs
-      : public std::unary_function< Type, Type > {
+      : public CGAL::cpp98::unary_function< Type, Type > {
       public:
         Type operator()( const Type& x ) const {
             return INTERN_INTERVAL_NT::abs( x );
@@ -967,7 +951,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class Sgn
-        : public std::unary_function< Type, Uncertain< ::CGAL::Sign > > {
+        : public CGAL::cpp98::unary_function< Type, Uncertain< ::CGAL::Sign > > {
       public:
         Uncertain< ::CGAL::Sign > operator()( const Type& x ) const {
             return INTERN_INTERVAL_NT::sign( x );
@@ -975,7 +959,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class Is_positive
-      : public std::unary_function< Type, Uncertain<bool> > {
+      : public CGAL::cpp98::unary_function< Type, Uncertain<bool> > {
       public:
         Uncertain<bool> operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::is_positive( x );
@@ -983,7 +967,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class Is_negative
-      : public std::unary_function< Type, Uncertain<bool> > {
+      : public CGAL::cpp98::unary_function< Type, Uncertain<bool> > {
       public:
         Uncertain<bool> operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::is_negative( x );
@@ -991,7 +975,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class Compare
-      : public std::binary_function< Type, Type, Comparison_result > {
+      : public CGAL::cpp98::binary_function< Type, Type, Comparison_result > {
       public:
       Comparison_result operator()( const Type& x, const Type& y ) const {
         return INTERN_INTERVAL_NT::compare( x, y );
@@ -1001,7 +985,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class To_double
-      : public std::unary_function< Type, double > {
+      : public CGAL::cpp98::unary_function< Type, double > {
       public:
         double operator()( const Type& x ) const {
             return INTERN_INTERVAL_NT::to_double( x );
@@ -1009,7 +993,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class To_interval
-      : public std::unary_function< Type, std::pair< double, double > > {
+      : public CGAL::cpp98::unary_function< Type, std::pair< double, double > > {
       public:
         std::pair<double, double> operator()( const Type& x ) const {
             return INTERN_INTERVAL_NT::to_interval( x );
@@ -1017,7 +1001,7 @@ template< bool B > class Real_embeddable_traits< Interval_nt<B> >
     };
 
     class Is_finite
-      : public std::unary_function< Type, Boolean > {
+      : public CGAL::cpp98::unary_function< Type, Boolean > {
       public :
         Boolean operator()( const Type& x ) const {
           return CGAL_NTS is_finite( x.inf() ) && CGAL_NTS is_finite( x.sup() );
@@ -1038,7 +1022,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     typedef Uncertain<bool>     Boolean; 
 
     class Is_zero
-      : public std::unary_function< Type, Boolean > {
+      : public CGAL::cpp98::unary_function< Type, Boolean > {
       public:
         Boolean operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::is_zero( x );
@@ -1046,7 +1030,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     };
 
     class Is_one
-      : public std::unary_function< Type, Boolean > {
+      : public CGAL::cpp98::unary_function< Type, Boolean > {
       public:
         Boolean operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::is_one( x );
@@ -1054,7 +1038,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     };
 
     class Square
-      : public std::unary_function< Type, Type > {
+      : public CGAL::cpp98::unary_function< Type, Type > {
       public:
         Type operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::square( x );
@@ -1062,7 +1046,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     };
 
     class Sqrt
-      : public std::unary_function< Type, Type > {
+      : public CGAL::cpp98::unary_function< Type, Type > {
       public:
         Type operator()( const Type& x ) const {
           return INTERN_INTERVAL_NT::sqrt( x );
@@ -1070,7 +1054,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     };
 
     struct Is_square
-        :public std::binary_function<Interval_nt<B>,Interval_nt<B>&,Boolean >
+        :public CGAL::cpp98::binary_function<Interval_nt<B>,Interval_nt<B>&,Boolean >
     {
         Boolean operator()(const Interval_nt<B>& x) const {
             return INTERN_INTERVAL_NT::is_positive( x );
@@ -1093,7 +1077,7 @@ class Algebraic_structure_traits< Interval_nt<B> >
     };
 
   class Divides
-    : public std::binary_function< Type, Type, Boolean > { 
+    : public CGAL::cpp98::binary_function< Type, Type, Boolean > {
   public:
     Boolean operator()( const Type& x, const Type&) const {
       return ! Is_zero()(x);
@@ -1154,86 +1138,86 @@ public:
   typedef CGAL::Tag_false With_empty_interval; 
   typedef CGAL::Tag_true  Is_interval; 
 
- struct Construct :public std::binary_function<Bound,Bound,Interval>{
+ struct Construct :public CGAL::cpp98::binary_function<Bound,Bound,Interval>{
     Interval operator()( const Bound& l,const Bound& r) const {
       CGAL_precondition( l < r ); 
       return Interval(l,r);
     }
   };
 
-  struct Lower :public std::unary_function<Interval,Bound>{
+  struct Lower :public CGAL::cpp98::unary_function<Interval,Bound>{
     Bound operator()( const Interval& a ) const {
       return a.inf();
     }
   };
 
-  struct Upper :public std::unary_function<Interval,Bound>{
+  struct Upper :public CGAL::cpp98::unary_function<Interval,Bound>{
     Bound operator()( const Interval& a ) const {
       return a.sup();
     }
   };
 
-  struct Width :public std::unary_function<Interval,Bound>{
+  struct Width :public CGAL::cpp98::unary_function<Interval,Bound>{
     Bound operator()( const Interval& a ) const {
       return width(a); 
     }
   };
 
-  struct Median :public std::unary_function<Interval,Bound>{
+  struct Median :public CGAL::cpp98::unary_function<Interval,Bound>{
     Bound operator()( const Interval& a ) const {
       return (Lower()(a)+Upper()(a))/2.0;
     }
   };
     
-  struct Norm :public std::unary_function<Interval,Bound>{
+  struct Norm :public CGAL::cpp98::unary_function<Interval,Bound>{
     Bound operator()( const Interval& a ) const {
       return magnitude(a);
     }
   };
 
-  struct Singleton :public std::unary_function<Interval,bool>{
+  struct Singleton :public CGAL::cpp98::unary_function<Interval,bool>{
     bool operator()( const Interval& a ) const {
       return Lower()(a) == Upper()(a);
     }
   };
 
-  struct Zero_in :public std::unary_function<Interval,bool>{
+  struct Zero_in :public CGAL::cpp98::unary_function<Interval,bool>{
     bool operator()( const Interval& a ) const {
       return Lower()(a) <= 0  &&  0 <= Upper()(a);
     }
   };
 
-  struct In :public std::binary_function<Bound,Interval,bool>{
+  struct In :public CGAL::cpp98::binary_function<Bound,Interval,bool>{
     bool operator()( Bound x, const Interval& a ) const {
       return Lower()(a) <= x && x <= Upper()(a);
     }
   };
 
-  struct Equal :public std::binary_function<Interval,Interval,bool>{
+  struct Equal :public CGAL::cpp98::binary_function<Interval,Interval,bool>{
     bool operator()( const Interval& a, const Interval& b ) const {
       return a.is_same(b);
     }
   };
     
-  struct Overlap :public std::binary_function<Interval,Interval,bool>{
+  struct Overlap :public CGAL::cpp98::binary_function<Interval,Interval,bool>{
     bool operator()( const Interval& a, const Interval& b ) const {
       return a.do_overlap(b);
     }
   };
     
-  struct Subset :public std::binary_function<Interval,Interval,bool>{
+  struct Subset :public CGAL::cpp98::binary_function<Interval,Interval,bool>{
     bool operator()( const Interval& a, const Interval& b ) const {
       return Lower()(b) <= Lower()(a) && Upper()(a) <= Upper()(b) ;  
     }
   };
     
-  struct Proper_subset :public std::binary_function<Interval,Interval,bool>{
+  struct Proper_subset :public CGAL::cpp98::binary_function<Interval,Interval,bool>{
     bool operator()( const Interval& a, const Interval& b ) const {
       return Subset()(a,b) && ! Equal()(a,b); 
     }
   };
     
-  struct Hull :public std::binary_function<Interval,Interval,Interval>{
+  struct Hull :public CGAL::cpp98::binary_function<Interval,Interval,Interval>{
     Interval operator()( const Interval& a, const Interval& b ) const {
       BOOST_USING_STD_MAX();
       BOOST_USING_STD_MIN();
@@ -1247,7 +1231,7 @@ public:
   
 //  struct Empty is Null_functor 
   
-  struct Intersection :public std::binary_function<Interval,Interval,Interval>{
+  struct Intersection :public CGAL::cpp98::binary_function<Interval,Interval,Interval>{
     Interval operator()( const Interval& a, const Interval& b ) const {
       BOOST_USING_STD_MAX();
       BOOST_USING_STD_MIN();
@@ -1284,6 +1268,13 @@ namespace Eigen {
       MulCost = 10
     };
   };
+
+  namespace internal {
+    template<class> struct significant_decimals_impl;
+    template<bool b>
+      struct significant_decimals_impl<CGAL::Interval_nt<b> >
+      : significant_decimals_impl<typename CGAL::Interval_nt<b>::value_type> { };
+  }
 }
 
 #endif // CGAL_INTERVAL_NT_H
